/*
 * Asterisk -- A telephony toolkit for Unix.
 *
 * Network Notification Application Module for Asterisk
 *
 * Copyright (c) 2005,2006 Sven Slezak <sunny@mezzo.net>
 * 1.0 (Apr23.2006)
 * http://www.mezzo.net/asterisk
 *
 */

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <sys/types.h>

#include <asterisk/options.h>
#include <asterisk/file.h>
#include <asterisk/logger.h>
#include <asterisk/channel.h>
#include <asterisk/pbx.h>
#include <asterisk/module.h>
#include <asterisk/astdb.h>
#include <asterisk/lock.h>
#include <asterisk/config.h>
#include <asterisk/utils.h>

#include <ctype.h>
#include <errno.h>

#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <sys/time.h> /* select() */ 

#define DEFAULT_PORT 40000
#define MAX_MSG 1024

int notify(char *text, char *host, int port);

static char *tdesc = "network notification function for Asterisk extension logic.";

static char *g_descrip =
  "  Notify(text/hostname_or_ip[:port]): \n"
  "   sends the text to the given hosts.\n"
  "   Always returns 0.\n";

static char *g_app = "Notify";

static char *g_synopsis = "network notification application";

STANDARD_LOCAL_USER;

LOCAL_USER_DECL;


static int get_exec (struct ast_channel *chan, void *data)
{
  int arglen, port;
  char *argv, *text, *host, *tmp;

  arglen = strlen(data);
  argv = alloca(arglen + 1);
  if(!argv) {
    ast_log (LOG_ERROR, "Memory allocation failed\n");
    return 0;
  }
  memcpy (argv, data, arglen + 1);

  if(strchr(argv, '/')) {
    text = strsep(&argv, "/");
    host = strsep(&argv, "\0");
    if (!text || !host) {
      ast_log (LOG_ERROR, "Ignoring: Syntax error in argument\n");
      return 0;
    } 
    if(strchr(host, ':')) {
      tmp = strsep(&host, ":");
      port = atoi(strsep(&host, "\0"));
      host = tmp;
    } else {
      port = DEFAULT_PORT;
    }
  } else {
    ast_log (LOG_ERROR, "Ignoring: Wrong parameters\n");
    return 0;
  }

  notify(text, host, port);

  return 0;
}

int unload_module(void) {
  int retval;
  STANDARD_HANGUP_LOCALUSERS;
  retval = ast_unregister_application(g_app);
  return retval;
}

int load_module(void) {
  int retval;
  retval = ast_register_application(g_app, get_exec, g_synopsis, g_descrip);
  return retval;
}

char *description(void) {
  return tdesc;
}

int usecount(void) {
  int res;
  STANDARD_USECOUNT (res);
  return res;
}

char *key() {
  return ASTERISK_GPL_KEY;
}


int notify(char *text, char *host, int port) {
  int sd, rc;
  struct sockaddr_in cliAddr, remoteServAddr;
  struct hostent *h;
  struct ast_hostent ahp;

  h = ast_gethostbyname(host, &ahp);
  if(h == (struct hostent *)0) {
    ast_log (LOG_ERROR, "unknown host: %s\n", host);
    return 0;
  }

  remoteServAddr.sin_family = h->h_addrtype;
  memcpy(&remoteServAddr.sin_addr, h->h_addr, sizeof(remoteServAddr.sin_addr));
  remoteServAddr.sin_port = htons(port);

  sd = socket(AF_INET,SOCK_DGRAM,0);
  if(sd<0) {
    ast_log (LOG_ERROR, "cannot open socket\n");
    return 0;
  }
  
  cliAddr.sin_family = AF_INET;
  cliAddr.sin_addr.s_addr = htonl(INADDR_ANY);
  cliAddr.sin_port = htons(0);
  rc = bind(sd, (struct sockaddr *) &cliAddr, sizeof(cliAddr));
  if(rc<0) {
    ast_log (LOG_ERROR, "cannot bind port: %d\n", port);
    close(sd);
    return 0;
  }

  if(option_verbose > 2)
    ast_verbose (VERBOSE_PREFIX_3 "Notify: sending '%s' to %s:%d \n", text, h->h_name?h->h_name:host, port);

  rc = sendto(sd, text, strlen(text)+1, 0, (struct sockaddr *) &remoteServAddr, sizeof(remoteServAddr));
  if(rc<0) {
    ast_log (LOG_ERROR, "cannot send text %s\n", text);
    close(sd);
    return 0;
  }
  
  close(sd);
  return 1;

}
